#!/usr/bin/env python3

############################################################################
#Script Name    : B008001_gst_vdec_buffer_compare.py
#Description    : Sript for checking buffer content alignment
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

TEST_PATTERN_DIR = '/home/root/genio-linux-test/test_pattern/008_vdec'

def subcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=5)
    return ret

def gen_md5(colorimetry,decoder):

    out_md5 = subcmd(["md5sum {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
    golden_md5 = subcmd(["md5sum {2}/golden-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])

    if(out_md5.stdout.split()[0] != golden_md5.stdout.split()[0]):
        ret_md5 = subcmd(["awk -i inplace -F, '/Got EOS from element \"pipeline0\"./ {{while (getline>0) ; next}} 1' {2}/out-320x320-{1}-{0}.md5 && echo \"$(<{2}/out-320x320-{1}-{0}.md5 )\" | sed '/^0:00:00.000000000/,$!d' | grep -v -w GstSystemClock | cut -f 2- -d ' ' > {2}/out-320x320-{1}-{0}.md5.out && mv {2}/out-320x320-{1}-{0}.md5.out {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
        out_md5 = subcmd(["md5sum {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
    else:
        ret_md5 = 0
    return ret_md5, out_md5, golden_md5

def runcmd(colorimetry,decoder):
    
    file_path = "{2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)
    if os.path.exists(file_path):
        ret_md5, out_md5, golden_md5 = gen_md5(colorimetry,decoder) 
    else:
        if decoder == 'vp8' or decoder == 'vp9':
            ret = subcmd(["gst-launch-1.0 -v filesrc location={2}/320x320-{1}-{0}.webm ! parsebin ! v4l2{1}dec ! v4l2convert ! checksumsink hash=0 sync=false > {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
        else:
            ret = subcmd(["gst-launch-1.0 -v filesrc location={2}/320x320-{1}-{0}.mp4 ! parsebin ! v4l2{1}dec ! v4l2convert ! video/x-raw,format={0} ! checksumsink hash=0 sync=false > {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
        if ret.returncode !=0:
            out_md5=golden_md5=0
        else:
            ret_md5, out_md5, golden_md5 = gen_md5(colorimetry,decoder)
    return ret_md5, out_md5, golden_md5
    
class TestGstBufferConsistency(unittest.TestCase):

    def test_buffer_consistency_h264_i420(self):     
        ret,out_md5,golden_md5 = runcmd("I420","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h264_yu12(self):
        ret,out_md5,golden_md5 = runcmd("YV12","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h264_nv12(self):
        ret,out_md5,golden_md5 = runcmd("NV12","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h264_nv21(self):
        ret,out_md5,golden_md5 = runcmd("NV21","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h264_rgb(self):
        ret,out_md5,golden_md5 = runcmd("RGB","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])
    
    def test_buffer_consistency_h264_argb(self):
        ret,out_md5,golden_md5 = runcmd("ARGB","h264")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h265_i420(self):     
        ret,out_md5,golden_md5 = runcmd("I420","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h265_yu12(self):
        ret,out_md5,golden_md5 = runcmd("YV12","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h265_nv12(self):
        ret,out_md5,golden_md5 = runcmd("NV12","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h265_nv21(self):
        ret,out_md5,golden_md5 = runcmd("NV21","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])
    
    def test_buffer_consistency_h265_rgb(self):
        ret,out_md5,golden_md5 = runcmd("RGB","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_h265_argb(self):
        ret,out_md5,golden_md5 = runcmd("ARGB","h265")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])
        
    def test_buffer_consistency_vp8_yuv(self):
        ret,out_md5,golden_md5 = runcmd("YUV","vp8")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])

    def test_buffer_consistency_vp9_yuv(self):
        ret,out_md5,golden_md5 = runcmd("YUV","vp9")
        self.assertEqual(out_md5.stdout.split()[0], golden_md5.stdout.split()[0])


if __name__ == '__main__':
    
    unittest.main()
