#!/usr/bin/env python3

############################################################################
#Script Name    : F018001_spi.py
#Description    : Sript for testing SPI
#                 Covered test cases: C491586
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess
from pathlib import Path

BASE = '/dev/spidev0.0'
mt8395_BASE = '/dev/spidev1.0'

def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=1)
    return ret

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

soc = get_soc_name()

class TestSPI(unittest.TestCase):
    
    def test_sysfs_existence(self):
        if soc == 'mt8395':
            self.assertTrue(pathlib.Path(mt8395_BASE).is_char_device())
        else:
            self.assertTrue(pathlib.Path(BASE).is_char_device())
        
    def test_spi_content_consistency(self):
        if soc == 'mt8395':
            spi_ret = runcmd(["spidev_test -D /dev/spidev1.0 -s 400000 -i test.bin -v"])
        else:
            spi_ret = runcmd(["spidev_test -D /dev/spidev0.0 -s 400000 -i test.bin -v"])
        self.assertEqual(spi_ret.returncode, 0)
        
        packets = spi_ret.stdout.split('\n')
        for rx, tx in zip(packets[-4:-1], packets[-7:-4]):
            content = rx.split('|')
            self.assertEqual(content[1],  content[1])
            self.assertEqual(content[2],  content[2]) 
        
if __name__ == '__main__':
    
    path_to_file = 'test.bin'
    path = Path(path_to_file)

    if not path.is_file():
        subprocess.run(["dd if=/dev/random of=test.bin bs=96 count=1"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=1)
    unittest.main()