 #!/usr/bin/env python3

############################################################################
#Script Name    : F018002_serialcheck.py
#Description    : Sript for testing serialcheck
#                 On G1200: UART1, On G350/G700/G510:UART2 
#                 Covered test cases: C487533
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess
from pathlib import Path


def runcmd(command, self):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    self.assertEqual(ret.returncode, 0)
    self.assertEqual(ret.stdout.split('\n')[-2], MSG)


def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

soc = get_soc_name()

if soc == 'mt8395':
    CMD_BASE = 'serialcheck -d /dev/ttyS1 -f binary -m d -l 3 -b '
elif soc == 'mt8390' or soc == 'mt8365' or soc == 'mt8370':
    CMD_BASE = 'serialcheck -d /dev/ttyS2 -f binary -m d -l 3 -b '

MSG = 'cts: 0 dsr: 0 rng: 0 dcd: 0 rx: 12288 tx: 12288 frame 0 ovr 0 par: 0 brk: 0 buf_ovrr: 0'               
    
               
    
class TestSerialCheck(unittest.TestCase):

    def test_baudrate_3000000(self):
        cmd = CMD_BASE + '3000000'
        runcmd([cmd], self)

    def test_baudrate_2000000(self):
        cmd = CMD_BASE + '2000000'
        runcmd([cmd], self)
        
    def test_baudrate_921600(self):
        cmd = CMD_BASE + '921600'
        runcmd([cmd], self)
        
    def test_baudrate_576000(self):
        cmd = CMD_BASE + '576000'
        runcmd([cmd], self)
        
    def test_baudrate_460800(self):
        cmd = CMD_BASE + '460800'
        runcmd([cmd], self)
        
    def test_baudrate_230400(self):
        cmd = CMD_BASE + '230400'
        runcmd([cmd], self)
        
    def test_baudrate_115200(self):
        cmd = CMD_BASE + '115200'
        runcmd([cmd], self)
        
    def test_baudrate_57600(self):
        cmd = CMD_BASE + '57600'
        runcmd([cmd], self)
        
    def test_baudrate_38400(self):
        cmd = CMD_BASE + '38400'
        runcmd([cmd], self) 
        
    def test_baudrate_19200(self):
        cmd = CMD_BASE + '19200'
        runcmd([cmd], self)  
        
    def test_baudrate_9600(self):
        cmd = CMD_BASE + '9600'
        runcmd([cmd], self)
        
    def test_baudrate_4800(self):
        cmd = CMD_BASE + '4800'
        runcmd([cmd], self)
        
    def test_baudrate_2400(self):
        cmd = CMD_BASE + '2400'
        runcmd([cmd], self)
        
    def test_baudrate_1200(self):
        cmd = CMD_BASE + '1200'
        runcmd([cmd], self)  
        
    def test_baudrate_600(self):
        cmd = CMD_BASE + '600'
        runcmd([cmd], self)
        
    def test_baudrate_300(self):
        cmd = CMD_BASE + '300'
        runcmd([cmd], self)
    
    def test_baudrate_300(self):
        cmd = CMD_BASE + '110'
        runcmd([cmd], self)
        
        
if __name__ == '__main__':

    path_to_file = 'binary'
    path = Path(path_to_file)

    if not path.is_file():
        subprocess.run(["dd if=/dev/urandom of=binary count=1 bs=4096"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=1)
    unittest.main()

    
    
    
    
