#!/bin/bash

###################################################################
#Script Name    : camera_test.sh
#Description    : Sript for testing testing camera
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###################################################################


LOGDIR=./
INDIR=./
RANDOM=$(date +%s)

monkey=0


cmd()
{
    echo "" >> ${LOGDIR}/app.log
    echo "$*" >> ${LOGDIR}/app.log
    $*
    return $?
}

cmdbg()
{
    echo "" >> ${LOGDIR}/app.log
    echo "Run $*"
    echo "$*" >> ${LOGDIR}/app.log

    $* 2>&1 &
    return $?
}

control_c() 
{
    kill $(jobs -p)
    exit
}
trap control_c SIGINT

log_record(){

    if [ "$test_mode" = "keep_playing" ]; then
        while true; do
            sleep 30
            T=`date "+%y/%m/%d %H:%M:%S"`
            echo "[$T] dump"
            echo "===== $T =====" >> $LOGDIR/mem.log
            cat /proc/`pgrep camd`/status | grep VmHWM >> $LOGDIR/mem.log
            echo "===== $T =====" >> $LOGDIR/dmesg.log 
        done
    
    elif [ "$test_mode" =  "stress" ] ; then
        T=`date "+%y/%m/%d %H:%M:%S"`
        echo "[$T] dump"
        echo "===== $T =====" >> $LOGDIR/mem.log
        cat /proc/`pgrep camd`/status | grep VmHWM >> $LOGDIR/mem.log
        echo "===== $T =====" >> $LOGDIR/dmesg.log 
    fi
}

set_default()
{
    cmd "v4l2-ctl -d ${video_device} -c brightness=0"
	cmd "v4l2-ctl -d ${video_device} -c contrast=0"
	cmd "v4l2-ctl -d ${video_device} -c saturation=0"
	cmd "v4l2-ctl -d ${video_device} -c white_balance_automatic=1"
	cmd "v4l2-ctl -d ${video_device} -c white_balance_temperature=6500"
	cmd "v4l2-ctl -d ${video_device} -c auto_exposure=0"
	cmd "v4l2-ctl -d ${video_device} -c exposure=0"
	cmd "v4l2-ctl -d ${video_device} -c exposure_time_absolute=33000000"
	cmd "v4l2-ctl -d ${video_device} -c focus_automatic_continuous=0"
	cmd "v4l2-ctl -d ${video_device} -c focus_absolute=0"
	cmd "v4l2-ctl -d ${video_device} -c power_line_frequency=3"
	cmd "v4l2-ctl -d ${video_device} -c sharpness=0"
	cmd "v4l2-ctl -d ${video_device} -c iso=100"
    cmd "v4l2-ctl -d ${video_device} -c hue=0"
}



while [[ $# -gt 0 ]]; do
        case "$1" in
                --video)
                        video_device=$2
                        shift 2
                        ;;
                --stream)
                        stream_type=$2
                        shift 2
                        ;;
				--test_mode)
                        test_mode=$2
                        shift 2
                        ;;
                --stream_mode)
                        stream_mode=$2
                        shift 2
                        ;;
				--format)
						color_format=$2
						shift 2
						;;
                --res)
                        resolution=$2
                        shift 2
                        ;;
                --monkey)
                        monkey=$2
                        shift 2
                        ;;
                --sensor)
                        sensor=$2
                        shift 2
                        ;;
                --platform)
                        platform=$2
                        shift 2
                        ;;
                --help)
                        echo "Usage: -sh --video [iv] --stream [is] --mode [im] --format [if] --res [ir]"
                        echo ""
                        echo "Run the camera test"
                        echo ""
                        echo "Options:"
                        echo " --video            Video Device"
                        echo " --stream           Stream Type, e.g. Preview, Video, Capture"
                        echo " --test_mode        Test Mode, e.g. overall, single_control, reverse_control, keep_playing, stress"
                        echo " --stream_mode      Stream Mode, e.g. gstreamer, v4l2ctl"
                        echo " --format           Color Format, e.g. YUY2, NM12"
                        echo " --res              Stream Resolution, e.g. 640x480, 1940x1080"
                        echo " --monkey           Round to open/close test"
                        echo " --sensor           Tested camera source, e.g. raw, AR0430, AR0830, AR0830_2lane"
                        echo " --platform         Tested platform, e.g. G350, G1200, G700" 
                        exit 0
                        ;;
                *)
                        break
                        ;;
        esac
done

declare -a input_format=("YUY2" "NV12" "YV12" "RGB") #v4l2src do NOT suppot YV12
declare -a input_format_v4l2=("YUYV" "NM12" "YM21" "RGB3")
declare -a input_resolution=("640x360" "576x432" "640x480" "960x540" "928x696" "1024x768" "1280x720" "1440x1080" "1920x1080" "1856x1392" "2048x1536" "2104x1560" "3840x2160" "4000x3000")
declare -a capture_resolution=("320x240" "640x480" "1280x720" "1280x960" "1440x1088" "1920x1080" "1920x1088" "1920x1440" "2560x1920" "3840x2160" "4096x2304" "4000x3000")


# Remove dirty things
rm -rf ${LOGDIR}/result.log ${LOGDIR}/app.log ${LOGDIR}/mem.log ${LOGDIR}/dmesg.log 
touch ${LOGDIR}/result.log ${LOGDIR}/app.log ${LOGDIR}/mem.log ${LOGDIR}/dmesg.log

dmesg -w > $LOGDIR/dmesg.log &


# [set all control with the value other than default ] : all resolution*all format
if [ "$test_mode" = "overall" ]; then
	echo "Test Mode: Overall"
    echo "Set controls to default..."
       set_default
    
    echo "Set controls to arbitrary value..."
       cmd "v4l2-ctl -d ${video_device} -c brightness=0,contrast=0,saturation=0,white_balance_automatic=1,white_balance_temperature=6500,auto_exposure=0,exposure=0,exposure_time_absolute=33000000,focus_automatic_continuous=0,focus_absolute=0,power_line_frequency=3,sharpness=0,iso=100"
	
    id=`echo ${video_device} | perl -F/ -wane 'print $F[-1]'`
    echo "Test for ${stream_type} device: "
	
    if [ "${stream_type}" = "Capture" ]; then
        for ir in ${capture_resolution[@]}; do
            IFS=x read iw ih <<< "${ir}"     
			echo "Test: ${stream_type}-${id}-JPEG-${ir} with V4L2-CTL:"

            cmd "v4l2-ctl --set-fmt-video=width=${iw},height=${ih},pixelformat=JPEG,sizeimage=4194304 --stream-mmap=1 --stream-count=1 --stream-to=/mnt/cam_out/${stream_type}-${id}-JPEG-${ir}.jpg -d ${video_device}"
			retVal=$?
            if [ $retVal -eq 0 ]; then
                echo "${stream_type} ${id} V4L2-CTL JPEG ${ir} : Pass" >> ${LOGDIR}/result.log
            else
                echo "${stream_type} ${id} V4L2-CTL JPEG ${ir} : Fail" >> ${LOGDIR}/result.log
            fi
			
			echo "Test: ${stream_type}-${id}-JPEG-${ir} with GST-LAUNCH:"
            cmd "gst-launch-1.0 v4l2src device=${video_device} num-buffers=1 ! image/jpeg,width=${iw},height=${ih},format=JPEG ! jpegdec ! waylandsink sync=false"
            retValGst=$?
            if [ $retValGst -eq 0 ]; then
				echo "${stream_type} ${id} Gstreamer JPEG ${ir} : Pass" >> ${LOGDIR}/result.log
			else
				echo "${stream_type} ${id} Gstreamer JPEG ${ir} : Fail" >> ${LOGDIR}/result.log
			fi
        done   
    elif [ "$stream_type" =  "Preview" ] || [ "$stream_type" =  "Video" ]; then
        for ir in ${input_resolution[@]}; do
            for if_idx in ${!input_format[@]}; do
				
                if=${input_format[$if_idx]}
				ifv=${input_format_v4l2[$if_idx]}
                IFS=x read iw ih <<< "${ir}"
				
				echo "Test: ${stream_type}-${id}-${ifv}-${ir} with V4L2-CTL:"
                cmd "v4l2-ctl --set-fmt-video=width=${iw},height=${ih},pixelformat=${ifv} --stream-mmap=1 --stream-count=1 --stream-to=/mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv -d ${video_device}"

				retVal=$?
                if [ $retVal -eq 0 ]; then
                    echo "${stream_type} ${id} V4L2-CTL ${ifv} ${ir} : Pass" >> ${LOGDIR}/result.log
                else
                    echo "${stream_type} ${id} V4L2-CTL ${ifv} ${ir} : Fail" >> ${LOGDIR}/result.log
                fi

				if [ ${ifv} = "YUYV" ]; then
					cmd "ffmpeg -s ${ir} -pix_fmt yuyv422 -i /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.jpg"
				elif [ ${ifv} = "NM12" ]; then
					mod=$(($ih%32))
                    wmod=$(($iw%16))
                    pih=${ih}
                    piw=${iw}
                    if [ $mod -ne 0 ]; then
                        pih=$((ih+$((32-$mod))))
                    fi
                    if [ $wmod -ne 0 ]; then
                        piw=$((piw+$((16-$wmod))))
                    fi
                       
					if [ $mod -ne 0 ] || [ $wmod -ne 0 ]; then
                        
						cmd "ffmpeg -s ${piw}x${pih} -pix_fmt nv12 -i /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.jpg"
					else
						cmd "ffmpeg -s ${ir} -pix_fmt nv12 -i /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.jpg"
					fi
                    
                elif [ ${ifv} = "YM21" ]; then
					cmd "ffmpeg -s ${ir} -pix_fmt yuv420p -i /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv -vf swapuv /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.jpg"
				elif [ ${ifv} = "rgb24" ]; then
					cmd "ffmpeg -s ${ir} -pix_fmt rgb24 -i /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.yuv /mnt/cam_out/${stream_type}-${id}-${ifv}-${ir}.jpg"
				fi
				
														
				echo "Test: ${stream_type}-${id}-${if}-${ir} with GST-LAUNCH:"
				
				if [ ${if} = "YUY2" ]; then
					cmd "gst-launch-1.0 v4l2src device=${video_device} num-buffers=1 ! video/x-raw,width=${iw},height=${ih},format=${if} ! waylandsink sync=false"
				elif [ ${if} = "NV12" ]; then
                    cmd "gst-launch-1.0 v4l2src device=${video_device} num-buffers=1 ! video/x-raw,width=${iw},height=${ih},format=${if} ! v4l2convert ! waylandsink sync=false"
                elif [ ${if} = "YV12" ]; then
					echo "V4l2src do NOT suppot YV12, skip test"
				elif [ ${if} = "RGB" ]; then
					cmd "gst-launch-1.0 v4l2src device=${video_device} num-buffers=1 ! video/x-raw,width=${iw},height=${ih},format=${if} ! videoconvert ! waylandsink sync=false"
				fi
				
                retValGst=$?
                if [ $retValGst -eq 0 ]; then
                    echo "${stream_type} ${id} Gstreamer ${if} ${ir} : Pass" >> ${LOGDIR}/result.log
                else
                    echo "${stream_type} ${id} Gstreamer ${if} ${ir} : Fail" >> ${LOGDIR}/result.log
                fi
            done
        done   
    fi

    echo ""
    echo "---------- Test Done ----------"
    echo "Result Failed:"
    cat ${LOGDIR}/result.log | grep -v "Pass"
    echo "-------------------------------"
fi    


# [basic control test] * all format *1080P
if [ "$test_mode" = "single_control" ] || [ "$test_mode" =  "reverse_control" ]; then
	echo "Test Mode: Single Control"
	   
	id=`echo ${video_device} | perl -F/ -wane 'print $F[-1]'`
    echo "Test for ${stream_type} device: "
	
    if [ "${stream_type}" = "Capture" ]; then
		echo "Test: ${stream_type}-${id}-JPEG-1920x1080 with GST-LAUNCH:"
		cmd "gst-launch-1.0 v4l2src device=${video_device} ! image/jpeg,width=1920,height=1080,format=JPEG ! jpegdec ! waylandsink sync=false"
	
	elif [ "$stream_type" =  "Preview" ] || [ "$stream_type" =  "Video" ]; then		
		echo "Test: ${stream_type}-${id}-${color_format}-1920x1080 with GST-LAUNCH:"
		if [ ${color_format} = "YUY2" ]; then
			cmd "gst-launch-1.0 v4l2src device=${video_device} ! video/x-raw,width=1920,height=1080,format=YUY2 ! waylandsink sync=false"
		elif [ ${color_format} = "NV12" ]; then
            cmd "gst-launch-1.0 v4l2src device=${video_device} ! video/x-raw,width=1920,height=1080,format=NV12 ! v4l2convert ! waylandsink sync=false"
        elif [ ${color_format} = "YV12" ]; then
			echo "V4l2src do NOT suppot YV12, skip test"
		elif [ ${color_format} = "RGB" ]; then
			cmd "gst-launch-1.0 v4l2src device=${video_device} ! video/x-raw,width=1920,height=1080,format=RGB ! videoconvert ! waylandsink sync=false"
		fi
	fi
fi


if [ "$test_mode" = "keep_playing" ]; then

    IFS=x read iw ih <<< "${resolution}"
    if [ "$sensor" = "raw" ] ; then
        echo "Run a concurrent pipeline on resolution=${iw}x${ih}, format=yuy2 for 24 hours"
        echo "You can terminate this test anytime with Ctrl+C"
    
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 3 | grep video | tr -d "\n"`) 
        echo "Preview node=${video[0]}, Video node=${video[1]}, Capture node=${video[2]}"
    
        echo -e "\nCheck the content of each lane is normal...each pipeline will launch for 3 seconds."
        echo -e "\nCheck preview lane..."
        cmd "gst-launch-1.0 v4l2src device=${video[0]} num-buffers=100 ! video/x-raw,width=${iw}, height=${ih},format=YUY2 ! waylandsink sync=false"
        echo -e "\nCheck video lane..."
        cmd "gst-launch-1.0 v4l2src device=${video[1]} num-buffers=100 ! video/x-raw,width=${iw}, height=${ih},format=YUY2 ! waylandsink sync=false"
        echo -e "\nCheck capture lane..."
        cmd "gst-launch-1.0 v4l2src device=${video[2]} num-buffers=100 ! image/jpeg,width=${iw},height=${ih},format=JPEG ! jpegdec ! waylandsink sync=false"
    
        echo -e "\nRun 3 lanes concurrently..."
        echo "Run the preview lane"
        cmdbg "gst-launch-1.0 v4l2src device=${video[0]} num-buffers=2592000 ! video/x-raw,width=${iw}, height=${ih},format=YUY2 ! waylandsink sync=false"
        echo "Run the video lane"
        cmdbg "gst-launch-1.0 v4l2src device=${video[1]} num-buffers=2592000 ! video/x-raw,width=${iw}, height=${ih},format=YUY2 ! waylandsink sync=false"
        echo "Run the capture lane"
        cmdbg "gst-launch-1.0 v4l2src device=${video[2]} num-buffers=2592000 ! image/jpeg,width=${iw},height=${ih},format=JPEG ! jpegdec ! waylandsink sync=false"
    
    elif [ "$sensor" = "AR0430" ] ; then
        echo "Run a pipeline on resolution=${iw}x${ih}, format=UYVY for 24 hours"
        echo "You can terminate this test anytime with Ctrl+C"
        
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 1 | grep video | tr -d "\n"`)
        if [ "$platform" = "G350" ] ; then
            video=(`v4l2-ctl --list-devices | grep mtk-camsv-isp30 -A 1 | grep video | tr -d "\n"`)   
        fi
        echo "Video node=${video[0]}"
        cmdbg "gst-launch-1.0 v4l2src device=${video[0]} num-buffers=2592000 ! video/x-raw,width=${iw},height=${ih},format=UYVY ! queue ! v4l2convert output-io-mode=dmabuf-import ! video/x-raw,width=1280,height=720 ! waylandsink sync=false"       
        
    elif [ "$sensor" = "AR0830" ] || [ "$sensor" = "AR0830_2lane" ]; then
        echo "Run a pipeline on resolution=${iw}x${ih}, format=UYVY for 24 hours"
        echo "You can terminate this test anytime with Ctrl+C"
        
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 1 | grep video | tr -d "\n"`)
        echo "Video node=${video[0]}"
        cmdbg "gst-launch-1.0 v4l2src device=${video[0]} num-buffers=2592000 ! video/x-raw,width=${iw},height=${ih},format=UYVY ! queue ! v4l2convert output-io-mode=dmabuf-import ! video/x-raw,width=1280,height=720 ! waylandsink sync=false"
    fi    

    log_record
fi

declare -a stream_id=(0 0 0)
stream_idy=0
if [ "$stream_mode" = "gstreamer" ] ; then
        declare -a input_format_preview=("YUY2" "NV12" "RGB")
        declare -a input_format_capture=("JPEG")
else
        declare -a input_format_preview=("YUYV" "NM12" "YM21" "RGB3")
        declare -a input_format_capture=("JPEG")
fi

declare -a input_resolution_ar0830=("1920x1080" "3200x1800" "3840x2160")
declare -a input_resolution_ar0830_2lane=("1920x1080" "2560x1440" "3840x2160")
declare -a stream_name=("Preview" "Video" "Capture")

if [ "$test_mode" = "stress" ]; then
    if [ "$sensor" = "raw" ] ; then
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 3 | grep video | tr -d "\n"`)    
        if [ $monkey -ne 0 ] ; then
            round=0
            while [ $round -lt $monkey ] ; do
                    echo "--- Round : ${round} ---"
                    r=$(($RANDOM%3))
                    echo "${stream_name[$r]} stream is opened."
                    if [ "${stream_id[$r]}" = "0" ] ; then
                            if [ "$r" = "0" ] || [ "$r" = "1" ] ; then
                                    IFS=x read iw ih <<< "${input_resolution[$(($RANDOM%${#input_resolution[@]}))]}"
                                    if=${input_format_preview[$(($RANDOM%${#input_format_preview[@]}))]}
                            elif [ "$r" = "2" ] ; then
                                    IFS=x read iw ih <<< "${capture_resolution[$(($RANDOM%${#capture_resolution[@]}))]}"
                                    if=${input_format_capture[$(($RANDOM%${#input_format_capture[@]}))]}
                            fi
    
                            if [ "$stream_mode" = "gstreamer" ] ; then
                                    if [ "$r" = "0" ] || [ "$r" = "1" ] ; then
                                        if [ ${if} = "YUY2" ]; then
                                            cmdbg "gst-launch-1.0 v4l2src device=${video[$r]} ! video/x-raw,width=${iw},height=${ih},format=${if} ! waylandsink sync=false"
                                        elif [ ${if} = "NV12" ]; then
                                            cmdbg "gst-launch-1.0 v4l2src device=${video[$r]} ! video/x-raw,width=${iw},height=${ih},format=${if} ! v4l2convert ! waylandsink sync=false"
                                        elif [ ${if} = "YV12" ]; then
                                            echo "V4l2src do NOT suppot YV12, skip test"
                                        elif [ ${if} = "RGB" ]; then
                                            cmdbg "gst-launch-1.0 v4l2src device=${video[$r]} ! video/x-raw,width=${iw},height=${ih},format=${if} ! videoconvert ! waylandsink sync=false"
                                        fi                       
                                    elif [ "$r" = "2" ] ; then
                                            cmdbg "gst-launch-1.0 v4l2src device=${video[$r]} ! image/jpeg,width=${iw},height=${ih},format=${if} ! jpegdec ! waylandsink sync=false"
                                    fi
                            else
                                    cmdbg "v4l2-ctl --set-fmt-video=width=${iw},height=${ih},pixelformat=${if} --stream-mmap=3 -d ${video[$r]}"
                            fi

                            stream_id[$r]=$!
                            echo "create pid = ${stream_id[$r]}"
                    else
                            echo "${stream_name[$r]} stream is closed."
                            echo "kill pid = ${stream_id[$r]}"
                            kill ${stream_id[$r]}
                            stream_id[$r]=0
                    fi
                    sleep 2
                    round=$(($round+1))
                    log_record
            done
            killall v4l2-ctl
            killall gst-launch-1.0
        fi
    elif [ "$sensor" = "AR0430" ] ; then
    
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 1 | grep video | tr -d "\n"`)
        if [ "$platform" = "G350" ] ; then
            video=(`v4l2-ctl --list-devices | grep mtk-camsv-isp30 -A 1 | grep video | tr -d "\n"`)   
        fi
        
        if [ $monkey -ne 0 ] ; then
            round=0
            while [ $round -lt $monkey ] ; do
                echo "--- Round : ${round} ---"

                if [ "$stream_mode" = "gstreamer" ] ; then
                    cmdbg "gst-launch-1.0 v4l2src device=${video[0]} ! video/x-raw,width=2316,height=1746,format=UYVY ! queue ! v4l2convert output-io-mode=dmabuf-import ! video/x-raw,width=1280,height=720 ! waylandsink sync=false"
                else
                    cmdbg "v4l2-ctl --set-fmt-video=width=2316,height=1746,pixelformat=UYVY --stream-mmap=3 -d ${video[0]}"
                fi

                stream_idy=$!
                echo "create pid = ${stream_idy}"
                sleep 7
                echo "kill pid = ${stream_idy}"
                kill ${stream_idy}
                stream_idy=0
                sleep 1

                round=$(($round+1))
                log_record
            done
            killall v4l2-ctl
            killall gst-launch-1.0
        fi
    elif [ "$sensor" = "AR0830" ] ; then
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 1 | grep video | tr -d "\n"`)
        if [ $monkey -ne 0 ] ; then
            round=0
            while [ $round -lt $monkey ] ; do
                echo "--- Round : ${round} ---"
                IFS=x read iw ih <<< "${input_resolution_ar0830[$(($RANDOM%${#input_resolution_ar0830[@]}))]}"
                if [ "$stream_mode" = "gstreamer" ] ; then
                    cmdbg "gst-launch-1.0 v4l2src device=${video[0]} ! video/x-raw,width=${iw},height=${ih},format=UYVY ! queue ! v4l2convert output-io-mode=dmabuf-import ! video/x-raw,width=1280,height=720 ! waylandsink sync=false"
                else
                    cmdbg "v4l2-ctl --set-fmt-video=width=${iw},height=${ih},pixelformat=UYVY --stream-mmap=3 -d ${video[0]}"
                fi
                stream_idy=$!
                echo "create pid = ${stream_idy}"
                sleep 7
                echo "kill pid = ${stream_idy}"
                kill ${stream_idy}
                stream_idy=0
                sleep 1
                round=$(($round+1))
                log_record
            done
            killall v4l2-ctl
            killall gst-launch-1.0
        fi
    elif [ "$sensor" = "AR0830_2lane" ] ; then
        declare -a video=(`v4l2-ctl --list-devices | grep mtk-v4l2-camera -A 1 | grep video | tr -d "\n"`)
        if [ $monkey -ne 0 ] ; then
            round=0
            while [ $round -lt $monkey ] ; do
                echo "--- Round : ${round} ---"
                IFS=x read iw ih <<< "${input_resolution_ar0830_2lane[$(($RANDOM%${#input_resolution_ar0830_2lane[@]}))]}"
                if [ "$stream_mode" = "gstreamer" ] ; then
                    cmdbg "gst-launch-1.0 v4l2src device=${video[0]} ! video/x-raw,width=${iw},height=${ih},format=UYVY ! queue ! v4l2convert output-io-mode=dmabuf-import ! video/x-raw,width=1280,height=720 ! waylandsink sync=false"
                else
                    cmdbg "v4l2-ctl --set-fmt-video=width=${iw},height=${ih},pixelformat=UYVY --stream-mmap=3 -d ${video[0]}"
                fi
                stream_idy=$!
                echo "create pid = ${stream_idy}"
                sleep 7
                echo "kill pid = ${stream_idy}"
                kill ${stream_idy}
                stream_idy=0
                sleep 1
                round=$(($round+1))
                log_record
            done
            killall v4l2-ctl
            killall gst-launch-1.0
        fi
    fi 
fi
