#!/bin/bash

###########################################################################
#Script Name    : F008001_video_decoding.sh
#Description    : Sript for decoding video
#                 Covered test cases: C3094,C3095,C3096,C3097,C3098
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
##########################################################################


LOGDIR=./

run()
{   
    IFS=, read format dec path <<< $(echo $*)
    vname=$(echo ${path} | rev | cut -d '/' -f 1 | rev)
    path="$(echo $path | sed 's/\ /\\ /g')"
    echo "Now Playing: ${path} ..."
    echo "" >> ${LOGDIR}/log.txt
    
	if [ ${format} == 'JPEG' ]; then
        echo "gst-launch-1.0 -v filesrc location=${path} ! jpegparse ! ${dec} ! queue ! v4l2convert output-io-mode=dmabuf-import capture-io-mode=dmabuf ! queue ! waylandsink" >> ${LOGDIR}/log.txt
        gst-launch-1.0 -v filesrc location=${path} ! jpegparse ! ${dec} ! queue ! v4l2convert output-io-mode=dmabuf-import capture-io-mode=dmabuf ! queue ! waylandsink >> ${LOGDIR}/log.txt 2>&1

    else 
        echo "gst-launch-1.0 -v filesrc location=${path} ! parsebin ! ${dec} ! queue ! v4l2convert output-io-mode=dmabuf-import capture-io-mode=dmabuf ! queue ! waylandsink" >> ${LOGDIR}/log.txt
        gst-launch-1.0 -v filesrc location=${path} ! parsebin ! ${dec} ! queue ! v4l2convert output-io-mode=dmabuf-import capture-io-mode=dmabuf ! queue ! waylandsink >> ${LOGDIR}/log.txt 2>&1
    fi

    if [ $? -eq 0 ]; then
        echo "${format}: ${vname}: Pass" >> ${LOGDIR}/result.txt
    else
        echo "${format}: ${vname}: Fail" >> ${LOGDIR}/result.txt
    fi
}


while [[ $# -gt 0 ]]; do
        case "$1" in
                --in-dir)
                        INDIR=$2
                        shift 2
                        ;;
                        
                --codec)
                        CODEC=$2
                        shift 2
                        ;;
                        
                --video)
                       VIDEO=$2
                       shift 2
                       ;;

                --soc)
                       SOC=$2
                       shift 2
                       ;;
                              
                --help)
                        echo "Usage: -sh --in-dir [id] "
                        echo ""
                        echo "Run the video test"
                        echo ""
                        echo "Options:"
                        echo " --in-dir        Input Directory of Sample Test VIDEO Pattern "
                        echo " --video         Play specific video file"
                        echo " --codec         Codec for in-dir or file. e.g. all, h264, h265, vp8, vp9, mpeg4, jpeg"
                        echo " --soc           Specify soc to be tested. e.g. MT8365, MT8390, MT8395"
                        exit 0 
                        ;;
                *)
                        break
                        ;;
        esac
done


# Remove dirty things
rm -rf ${LOGDIR}/result.txt ${LOGDIR}/log.txt
touch ${LOGDIR}/result.txt ${LOGDIR}/log.txt


declare -a input_codec=("H264" "H265" "VP8" "VP9" "MPEG4" "JPEG") 

if  [ ! -z "${VIDEO}" ]; then
    if  [ -z ${CODEC} ]; then
        echo -e  "[Error]: Please specify Codec by --codec !!!\n"
    else
        run "${CODEC},v4l2${CODEC}dec,${VIDEO}"
    fi
else
    if  [ -z "${INDIR}" ]; then
        echo -e  "[Error]: Please set test pattern root directory by --in-dir, or set video by --video !!!\n"
    else
        if  [ -z ${CODEC} ]; then
            echo -e  "[Error]: Please specify Codec by --codec !!!\n"
        elif [ ${CODEC} = 'all' ]; then
            if  [ -z ${SOC} ]; then
                echo -e  "[Error]: Please specify SOC by --soc !!!\n"
            else
                for ic in ${input_codec[@]}; do
                if [ ${ic} == 'H264' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2h264dec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2h264dec,${FILE}"
                        done
                    fi
                elif [ ${ic} = 'H265' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2h265dec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2h265dec,${FILE}"
                        done
                    fi
                elif [ ${ic} = 'VP8' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2hvp8dec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2hvp8dec,${FILE}"
                        done
                    fi
                elif [ ${ic} = 'VP9' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2vp9dec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2vp9dec,${FILE}"
                        done
                    fi
            
                elif [ ${ic} = 'MPEG4' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2mpeg4dec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2mpeg4dec,${FILE}"
                        done
                    fi
                elif [ ${ic} = 'JPEG' ]; then
                    if [ ${SOC} == 'MT8365' ]; then
                        find ${INDIR}/${ic} -maxdepth 1 -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2jpegdec,${FILE}"
                        done
                    else
                        find ${INDIR}/${ic} -type f -print | while IFS= read -r FILE;
                        do
                        #echo -e "${FILE}\n"
                        run "${ic},v4l2jpegdec,${FILE}"
                        done
                    fi
                fi
                done
            fi  
        else
            find ${INDIR} -type f -print | while IFS= read -r FILE;
            do
                    #echo -e "${FILE}\n"
                    run "${CODEC},v4l2${CODEC}dec,${FILE}"
            done  
        fi
    fi    
fi
              
# Print result
echo ""
echo "---------- Test Done ----------"
echo "Result Failed:"
cat ${LOGDIR}/result.txt | grep -v "Pass"
echo "-------------------------------"
echo "Note: The result only indicates gstreamer executes successfully or not."
echo "      You still need to check the quality of ouput videos."
