#!/bin/bash

###########################################################################
#Script Name    : F011001_audio_stress_test.sh
#Description    : Sript for repeating audio playback
#                 Covered test cases: C6187,C6188 
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
##########################################################################

# Usage : sh ./F011001_audio_stress_test.sh COMMAND
# e.g.
#     # command="aplay sample-test-pattern-main/AUDIO/WAV/file-example-WAV-10MG.wav"
#     # sh ./F011001_audio_stress_test.sh $command
# or
#     # sh ./F011001_audio_stress_test.sh aplay sample.wav

LOGDIR=./
STOP=0
trap "STOP=1" SIGINT
command=$*
process=$1


# Remove dirty things
rm -rf ${LOGDIR}/app.log ${LOGDIR}/app.log ${LOGDIR}/mem.log ${LOGDIR}/dmesg.log 
touch ${LOGDIR}/app.log ${LOGDIR}/app.log ${LOGDIR}/mem.log ${LOGDIR}/dmesg.log

cmd()
{
	echo "Here is $*"
	eval $*
	if [ "$?" != 0 ] ; then
		echo "Fail"
		exit 1
	fi
}

run()
{
	# your gstreamer command comes here
	loop=0
	while [ "$STOP" != 1 ]; do
		echo "Loop ${loop}"
		cmd $command
		loop=$((loop+1))
	done
}

# Run gstreamer program in background
run > $LOGDIR/app.log &
shpid=$!

# Dump memory & kernel log
dmesg -w > $LOGDIR/dmesg.log &
dmesg=$!

while [ "$STOP" != 1 ]; do
	sleep 30
	T=`date "+%y/%m/%d %H:%M:%S"`
	echo "[$T] dump"
	echo "===== $T =====" >> $LOGDIR/mem.log
    echo "${process}" >> $LOGDIR/mem.log
    cat /proc/`pgrep vpud`/status | grep VmHWM >> $LOGDIR/mem.log
	echo "===== $T =====" >> $LOGDIR/dmesg.log
done

# Stop shell
kill $shpid
kill $dmesg

# Stop process command
PID=`pidof $process`
if [ -n "$PID" ]; then
	kill $PID
fi

echo "Quit"
