#!/bin/bash

###########################################################################
#Script Name    : F009001_machine_learning.sh
#Description    : Sript for testing machine learning related functionality
#                 Covered test cases: C1087,C3270,C3271,C3272,C3273,C5932
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
##########################################################################


LABEL_IMAGE="/usr/share/tensorflow-lite/label_image/label_image.py"
LABEL="/usr/share/tensorflow-lite/label_image/labels_mobilenet_quant_v1_224.txt"
MODEL="/usr/share/tensorflow-lite/label_image/mobilenet_v1_1.0_224_quant.tflite"
IMAGE="/usr/share/tensorflow-lite/label_image/grace_hopper.jpg"
TEST_HOME="/home/ubuntu/test"

SOC=`cat /proc/device-tree/compatible | tr ',' ' ' | tr '-' ' ' | gawk '{print toupper($2)}'`

if  [ -z ${SOC} ]; then
    echo -e  "[Error]: SOC is not defined !!!\n"
else
    echo -e "\033[33mTest for Tesorflow Lite CPU Delegate:\n\033[0m"
    echo -e "\033[33mRun Label Image Test:\033[0m"
    python3 ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL}
    echo -e "\033[33m \nPlease check the result for Label Image Test!\033[0m" 
    echo -e "\n====================================\n"
    read -n 1 -s -r -p "Press any key to continue..."

    echo -e "\033[33m\nTest for Tesorflow Lite GPU Delegate:\n\033[0m"

    echo -e "\033[33mRun Label Image Test:\033[0m"
    python3  ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL} -e /usr/lib/aarch64-linux-gnu/gpu_external_delegate.so
    echo -e "\033[33m \nPlease check the result for Label Image Test!\033[0m" 
    echo -e "\n====================================\n"
    read -n 1 -s -r -p "Press any key to continue..."

    echo -e "\033[33m \nTest for Tesorflow Lite ArmNN(CPU) Delegate:\n\033[0m"
    echo -e "\033[33mRun Label Image Test:\033[0m"
    python3 ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL} -e /usr/lib/aarch64-linux-gnu/libarmnnDelegate.so.28 -o "backends:CpuAcc,GpuAcc"
    echo -e "\033[33m \nPlease check the result for Label Image Test!\033[0m" 
    echo -e "\n====================================\n"
    read -n 1 -s -r -p "Press any key to continue..."

    echo -e "\033[33m \nTest for Tesorflow Lite ArmNN(GPU) Delegate:\n\033[0m"
    echo -e "\033[33mRun Label Image Test:\033[0m"
    python3 ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL} -e /usr/lib/aarch64-linux-gnu/libarmnnDelegate.so.28 -o "backends:GpuAcc,CpuAcc"
    echo -e "\033[33m \nPlease check the result for Label Image Test!\033[0m"
    echo -e "\n====================================\n"
    read -n 1 -s -r -p "Press any key to continue..."

    echo -e "\033[33m \nTest for Tesorflow Lite ArmNN(GPU) Delegate:\n\033[0m"
    echo -e "\033[33mRun Label Image Test:\033[0m"
    python3 ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL} -e /usr/lib/aarch64-linux-gnu/libarmnnDelegate.so.28 -o "backends:CpuRef,CpuAcc"
    echo -e "\033[33m \nPlease check the result for Label Image Test!\033[0m"
    echo -e "\n====================================\n"
    read -n 1 -s -r -p "Press any key to continue..."

    if [ ${SOC} == 'MT8365' ]; then
        echo -e "\033[33m \nTest for Android NNAPI Delegate(VP6)\n:\033[0m"
        echo -e "\033[33mRun Label Image Test:\033[0m"
        python3 ${LABEL_IMAGE} --label_file ${LABEL} --image ${IMAGE} --model_file ${MODEL} -e /usr/lib/aarch64-linux-gnu/nnapi_external_delegate.so
    else
        echo -e "\033[33m \nTest for NeuronSDK:\n\033[0m"
        echo -e "\033[33m \nRun Label Image Test:\033[0m"
        cd ${TEST_HOME}/neuropilot/demo_dla/ && sudo python3 label_image.py
    fi

    echo -e "\033[33mPlease check the result for Label Image Test!\033[0m" 
    echo -e "=============Test Done============\n"
fi


