#!/usr/bin/env python3

############################################################################
#Script Name    : S000001_emmc_partition.py
#Description    : Sript for testing Boot Device :: eMMC  
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess


PARTITION = ['bootloaders','bootloaders_b' ,'firmware','firmware_b' ,'dramk', 'misc','EFI_system_partition', 'bootassets', 'kernel', 'rootfs']

def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=1)
    return ret

def get_distributor():
    ret = subprocess.run(["lsb_release -si"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    distro = ret.stdout.split('\n')[0]
    return distro

distro = get_distributor()
uname = runcmd(["uname -a"]).stdout.split()[1]

@unittest.skipIf(distro == 'Ubuntu', "test is not compatible with Ubuntu")
@unittest.skipIf(distro == 'Debian', "test is not compatible with Debian")

class TestPartedBootDevice(unittest.TestCase):

    def test_sysfs_existence(self):
        if uname == 'genio-1200-evk-ufs':
            BASE = '/dev/sdc'
        else:
            BASE = '/dev/mmcblk0'

        self.assertTrue(pathlib.Path(BASE).is_block_device())

    def test_parted_ignore(self):
        if uname == 'genio-1200-evk-ufs':
            ret = runcmd(["printf ignore | parted /dev/sdc print"])
        else:        
            ret = runcmd(["printf ignore | parted /dev/mmcblk0 print"])
        
        self.assertEqual(ret.returncode, 0)
        for line in ret.stdout.split('\n')[9:-2]:
            self.assertIn(line.split(',')[0].split()[-2] , PARTITION)


if __name__ == '__main__':
    
    unittest.main()
