#!/usr/bin/env python3

############################################################################
#Script Name    : S004001_suspend_resume.py
#Description    : Sript for testing suspend to IDLE/RAM
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys
import unittest
import subprocess


def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=9)
    return ret

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

def get_distributor():
    ret = subprocess.run(["lsb_release -si"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    distro = ret.stdout.split('\n')[0]
    return distro

distro = get_distributor()

@unittest.skipIf(distro == 'Ubuntu', "test is not compatible with Ubuntu")
@unittest.skipIf(distro == 'Debian', "test is not compatible with Debian")

class TestSuspendResume(unittest.TestCase):
    def test_suspend_to_ram_5sec(self):
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            ret = runcmd(["echo '' > /sys/kernel/config/usb_gadget/g1/UDC && rtcwake -s 5 -m mem && echo 112b1000.usb > /sys/kernel/config/usb_gadget/g1/UDC"])
            self.assertEqual(ret.returncode, 0)
        elif soc == 'mt8395' or soc == 'mt8195':
            ret = runcmd(["modprobe -r tcpci_mt6360 && rtcwake -s 5 -m mem && modprobe tcpci_mt6360"])
            self.assertEqual(ret.returncode, 0)
        elif soc == 'mt8365':
            ret = runcmd(["echo '' > /sys/kernel/config/usb_gadget/g1/UDC && rtcwake -s 5 -m mem && echo 11201000.usb > /sys/kernel/config/usb_gadget/g1/UDC"])
            self.assertEqual(ret.returncode, 0)

if __name__ == '__main__':

    unittest.main()
