#!/usr/bin/env python3

##################################################################################
#Script Name    : S004002_cpuidle.py
#Description    : Sript for testing CPU Idle(WFI)
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
##################################################################################

import os, sys
import unittest
import subprocess
import time

BASE = '/sys/devices/system/cpu'

def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def read_attr_num(attr):
    tmp = read_attr(attr)
    if not tmp:
        return -1

    return int(tmp)

def write_attr(attr, value):
    path = os.path.join(BASE, attr)
    try:
        f = open(path, 'w')
        f.write(value)
        f.close()
    except IOError:
        return False

    return True


def read_idle_attr(cpu, state, attr):
    return read_attr('cpu%d/cpuidle/state%d/%s' % (cpu, state, attr))

def read_idle_attr_num(cpu, state, attr):
    return read_attr_num('cpu%d/cpuidle/state%d/%s' % (cpu, state, attr))
    
    
def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

soc = get_soc_name()
class TestCpuIdle(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_read_wfi(self):
        self.assertEqual(read_idle_attr(0, 0, 'name'), 'WFI')
        self.assertEqual(read_idle_attr(0, 0, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 0, 'usage'), 0)
        
    @unittest.skipIf(soc == 'mt8365', "not supported for %s" % soc)      
    def test_read_clusteroff_l(self):
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(read_idle_attr(0, 2, 'name'), 'clusteroff-l')
        if soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(read_idle_attr(0, 2, 'name'), 'clusteroff_l')
            
        self.assertEqual(read_idle_attr(0, 2, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 2, 'usage'), 0)
          
    @unittest.skipIf(soc == 'mt8365', "not supported for %s" % soc)  
    def test_read_clusteroff_b(self):
        if soc == 'mt8390':
            self.assertEqual(read_idle_attr(6, 2, 'name'), 'clusteroff-b')
            self.assertEqual(read_idle_attr(6, 2, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(6, 2, 'usage'), 0)
        
        elif soc == 'mt8370':
            self.assertEqual(read_idle_attr(4, 2, 'name'), 'clusteroff-b')
            self.assertEqual(read_idle_attr(4, 2, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(4, 2, 'usage'), 0)
        
        elif soc == 'mt8395' or soc == 'mt8195'or soc == 'mt8370':
            self.assertEqual(read_idle_attr(4, 2, 'name'), 'clusteroff_b')
            self.assertEqual(read_idle_attr(4, 2, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(4, 2, 'usage'), 0)
            
    @unittest.skipIf(soc == 'mt8365', "not supported for %s" % soc)      
    def test_read_cpuoff_l(self):
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(read_idle_attr(0, 1, 'name'), 'cpuoff-l')
        if soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(read_idle_attr(0, 1, 'name'), 'cpuoff_l')
            
        self.assertEqual(read_idle_attr(0, 1, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 1, 'usage'), 0)
        
    @unittest.skipIf(soc == 'mt8365', "not supported for %s" % soc)      
    def test_read_cpuoff_b(self):
        if soc == 'mt8390':
            self.assertEqual(read_idle_attr(6, 1, 'name'), 'cpuoff-b')
            self.assertEqual(read_idle_attr(6, 1, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(6, 1, 'usage'), 0)
        
        if soc == 'mt8370':
            self.assertEqual(read_idle_attr(4, 1, 'name'), 'cpuoff-b')
            self.assertEqual(read_idle_attr(4, 1, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(4, 1, 'usage'), 0)
            
        elif soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(read_idle_attr(4, 1, 'name'), 'cpuoff_b')
            self.assertEqual(read_idle_attr(4, 1, 'disable'), '0')
            self.assertGreater(read_idle_attr_num(4, 1, 'usage'), 0)
            
    @unittest.skipIf(soc != 'mt8365', "not supported for %s" % soc)  
    def test_read_mcdi_cluster(self):
        self.assertEqual(read_idle_attr(0, 2, 'name'), 'mcdi-cluster')
        self.assertEqual(read_idle_attr(0, 2, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 2, 'usage'), 0)
    
    @unittest.skipIf(soc != 'mt8365', "not supported for %s" % soc)  
    def test_read_dpidle(self):
        self.assertEqual(read_idle_attr(0, 3, 'name'), 'dpidle')
        self.assertEqual(read_idle_attr(0, 3, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 3, 'usage'), 0)
        
    @unittest.skipIf(soc != 'mt8365', "not supported for %s" % soc)   
    def test_read_mcdi_cpu(self):
        self.assertEqual(read_idle_attr(0, 1, 'name'), 'mcdi-cpu')
        self.assertEqual(read_idle_attr(0, 1, 'disable'), '0')
        self.assertGreater(read_idle_attr_num(0, 1, 'usage'), 0)
    
if __name__ == '__main__':
    unittest.main()
