#!/usr/bin/env python3

############################################################################
#Script Name    : S004003_cpuhotplug.py
#Description    : Sript for testing CPU Hotplug
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys
import unittest
import subprocess
import time

BASE = '/sys/devices/system/cpu'

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc
soc = get_soc_name()

def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def read_attr_num(attr):
    tmp = read_attr(attr)
    if not tmp:
        return -1

    return int(tmp)

def write_attr(attr, value):
    path = os.path.join(BASE, attr)
    try:
        f = open(path, 'w')
        f.write(value)
        f.close()
    except IOError:
        return False

    return True

def read_cpus():
    cpus = []

    i = 0
    while True:
        t = read_attr('cpu%d/online' % i)
        if not t:
            break

        cpus.append(t)
        i += 1

    return cpus

def read_cpu_online(cpu):
    return read_attr('cpu%d/online' % cpu)

def write_cpu_online(cpu, value):
    return write_attr('cpu%d/online' % cpu, value)

def read_cpuinfo():
    tmp = {}
    ret = subprocess.run('cat /proc/cpuinfo', shell=True, capture_output=True)
    if ret.returncode != 0:
        return None

    out = ret.stdout.decode('utf-8')
    lines = out.splitlines()
    for l in lines:
        if l.startswith('processor'):
            cpuid = l.split(': ')[1]
            tmp[cpuid] = True

    return tmp


class TestCpuHotplug(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_read_cpus(self):
        cpus = read_cpus()
        
        if soc == 'mt8365':
            self.assertEqual(len(cpus), 4)
        elif soc == 'mt8370':
            self.assertEqual(len(cpus), 6)
        else:
            self.assertEqual(len(cpus), 8)

    def test_cpu_offline(self):
        state = read_cpu_online(3)
        self.assertEqual(state, "1")

        # Turn off cpu3
        self.assertTrue(write_cpu_online(3, "0"))
        state = read_cpu_online(3)
        self.assertEqual(state, "0")
        cpuinfos = read_cpuinfo()
        self.assertNotEqual(cpuinfos, None)
        #print('cpuinfos: ', cpuinfos)
        self.assertTrue("3" not in cpuinfos)

        # Turn on cpu3
        self.assertTrue(write_cpu_online(3, "1"))
        state = read_cpu_online(3)
        self.assertEqual(state, "1")
        cpuinfos = read_cpuinfo()
        self.assertNotEqual(cpuinfos, None)
        #print('cpuinfos: ', cpuinfos)
        self.assertTrue("3" in cpuinfos)


if __name__ == '__main__':
    unittest.main()
