#!/usr/bin/env python3

############################################################################
#Script Name    : S004004_PMIC_regulator.py
#Description    : Sript for checking the number of PMIC regulator nodes 
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys
import unittest
import subprocess

BASE = '/sys/class/regulator'
MAIN_REGULATORS = ['vs1', 'vgpu11', 'vmodem', 'vpu', 'vcore', 'vs2', 'vpa', 'vproc2', 'vproc1', 'vgpu11_sshub', 'vaud18', 'vsim1', 'vibr', 'vrf12', 'vusb', 'vsram_proc2', 'vio18', 'vcamio', 'vcn18', 'vfe28', 'vcn13', 'vcn33_1_bt', 'vcn33_1_wifi', 'vaux18', 'vsram_others', 'vefuse', 'vxo22', 'vrfck', 'vbif28', 'vio28', 'vemc', 'vcn33_2_bt', 'vcn33_2_wifi', 'va12', 'va09', 'vrf18', 'vsram_md', 'vufs', 'vm18', 'vbbck', 'vsram_proc1', 'vsim2', 'vsram_others_sshub']
mt8365_MAIN_REGULATORS = ['vproc', 'vcore', 'vmodem', 'vs1', 'vpa', 'vfe28', 'vxo22', 'vrf18', 'vrf12', 'vefuse', 'vcn33-bt', 'vcn33-wifi', 'vcn28', 'vcn18', 'vcama', 'vcamd', 'vcamio', 'vldo28', 'vsram-others', 'vsram-proc', 'vaux18', 'vaud28', 'vio28', 'vio18', 'vdram', 'vmc', 'vmch', 'vemc', 'vsim1', 'vsim2', 'vibr', 'vusb33']

def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def read_attr_name(attr):
    tmp = read_attr(attr)
    if not tmp:
        return -1

    return tmp

def write_attr(attr, value):
    path = os.path.join(BASE, attr)
    try:
        f = open(path, 'w')
        f.write(value)
        f.close()
    except IOError:
        return False

    return True

def read_name(reg):
    return read_attr_name('regulator.%d/name' % reg)

def read_all_name():
    tmp = []

    i = 0
    while True:
        t = read_name(i)
        if t == -1:
            break

        tmp.append(t)
        i += 1

    return tmp
    
def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc
    
class TestPMICRegulator(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_sysfs_attrs_read(self):
        names = read_all_name()
        soc = get_soc_name()
        if soc == 'mt8365':
            for node in mt8365_MAIN_REGULATORS:
                self.assertIn(node ,names)
        else:
            for node in MAIN_REGULATORS:
                self.assertIn(node ,names)        

if __name__ == '__main__':
    unittest.main()
