#!/usr/bin/env python3

############################################################################
#Script Name    : S004005_linux_ccf.py
#Description    : Sript for testing Clock Manager(CCF)
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

BASE = '/sys/kernel/debug/clk'

def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=1)
    return ret

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

soc = get_soc_name()

@unittest.skipIf(soc == 'mt8365', "not supported mt8365")      
class TestLinuxCCF(unittest.TestCase):
    
    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))
        
    def test_linux_ccf(self):
        chmod_ret = runcmd(["chmod 777 -R ./sanity/004_power/linux_ccf"])
        self.assertEqual(chmod_ret.returncode, 0, chmod_ret.stderr)
        
        cat_ret = runcmd(["cat /sys/kernel/debug/clk/clk_summary > ./sanity/004_power/linux_ccf/clk-summary.txt"])
        self.assertEqual(cat_ret.returncode, 0)
        
        if soc == 'mt8390' or soc == 'mt8370':
            verify_ret = runcmd(["./sanity/004_power/linux_ccf/verify-mt8188-ccf.sh -t ./sanity/004_power/linux_ccf/mt8188-clk.h -s ./sanity/004_power/linux_ccf/clk-summary.txt"])
        elif soc == 'mt8395' or soc == 'mt8195':
            verify_ret = runcmd(["./sanity/004_power/linux_ccf/verify-mt8195-ccf.sh -t ./sanity/004_power/linux_ccf/mt8195-clk.h -s ./sanity/004_power/linux_ccf/clk-summary.txt"])
             
        self.assertEqual(verify_ret.returncode, 0)
        self.assertEqual(verify_ret.stdout.split('\n')[0], '[-] Success, all clocks are mapped !')
        
if __name__ == '__main__':
    
    unittest.main()
