#!/usr/bin/env python3

############################################################################
#Script Name    : S004006_thermal.py
#Description    : Sript for testing thermal control 
#                 Covered test cases: C490489
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys
import unittest
import subprocess

BASE = '/sys/class/thermal'

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc
soc = get_soc_name()

def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def read_attr_num(attr):
    tmp = read_attr(attr)
    if not tmp:
        return -1

    return int(tmp)

def write_attr(attr, value):
    path = os.path.join(BASE, attr)
    try:
        f = open(path, 'w')
        f.write(value)
        f.close()
    except IOError:
        return False

    return True

def read_temp(zone):
    return read_attr_num('thermal_zone%d/temp' % zone)

def read_all_temp():
    tmp = []

    i = 0
    while True:
        t = read_temp(i)
        if t == -1:
            break

        tmp.append(t)
        i += 1

    return tmp

def read_trip_type(zone, trip):
    return read_attr('thermal_zone%d/trip_point_%d_type' % (zone, trip))

def read_trip_temp(zone, trip):
    return read_attr_num('thermal_zone%d/trip_point_%d_temp' % (zone, trip))

def read_trips():
    # Currently only thermal zone 0 has trip points
    base = os.path.join(BASE, 'thermal_zone0')
    trips = {'zone0': []}

    i = 0
    while True:
        path = os.path.join(base, 'trip_point_%d_type' % i)
        if not os.path.exists(path):
            break

        t = {}
        t['type'] = read_trip_type(0, i)
        t['temp'] = read_trip_temp(0, i)
        trips['zone0'].append(t)          

        i += 1

    return trips           

def read_cdevs():
    # Currently only thermal zone 0 has cooling devices
    base = os.path.join(BASE, 'thermal_zone0')
    cdevs = {'zone0': []}

    i = 0
    while True:
        path = os.path.join(base, 'cdev%d' % i, 'type')
        if not os.path.exists(path):
            break

        try:
            f = open(path)
            cdev_type = f.read().strip()
            f.close()
            cdevs['zone0'].append(cdev_type)
        except IOError:
            psss

        i += 1

    return cdevs

class TestThermal(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_thermal_zones(self):
        temps = read_all_temp()
        
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(len(temps), 15)
        elif soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(len(temps), 18)
        elif soc == 'mt8365':
            self.assertEqual(len(temps), 1)
        
        for t in temps:
            self.assertGreater(t, 0, 'Temperature should be positive value')

    def test_trip_points(self):
        trips = read_trips()
        self.assertEqual(len(trips['zone0']), 3)

    def test_cooling_devices(self):
        cdevs = read_cdevs()
        
        if soc == 'mt8365':
            self.assertEqual(len(cdevs['zone0']), 1)
        else:
            self.assertEqual(len(cdevs['zone0']), 2)
        


if __name__ == '__main__':
    unittest.main()
