#!/usr/bin/env python3

############################################################################
#Script Name    : S004008_DVFS_gpufreq.py
#Description    : Sript for testing GPU frequency adjustment (MALI) 
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys
import unittest
import subprocess


mt8390_FREQS = '950000000 915000000 880000000 835000000 790000000 760000000 730000000 700000000 670000000 640000000 598000000 556000000 515000000 473000000 431000000 390000000'
mt8395_FREQS = '880000000 850000000 820000000 790000000 760000000 730000000 700000000 670000000 640000000 598000000 556000000 515000000 473000000 431000000 410000000 390000000'
mt8365_FREQS = '800000000 560000000 450000000'
GOVERNORS = ['userspace', 'powersave', 'performance', 'simple_ondemand']

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc
soc = get_soc_name()

if soc == 'mt8365':
    BASE = '/sys/devices/platform/soc/13040000.mali/devfreq/13040000.mali'
else:
    BASE = '/sys/devices/platform/soc/13000000.mali/devfreq/13000000.mali'


def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def read_attr_num(attr):
    tmp = read_attr(attr)
    if not tmp:
        return -1

    return int(tmp)

def write_attr(attr, value):
    path = os.path.join(BASE, attr)
    try:
        f = open(path, 'w')
        f.write(value)
        f.close()
    except IOError:
        return False

    return True

class TestGpuFreq(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_sysfs_attrs_read(self):
        for node in GOVERNORS:
            self.assertIn(node ,read_attr('available_governors').split())
        self.assertNotEqual(read_attr_num('cur_freq'), -1)
        
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(read_attr('available_frequencies'), mt8390_FREQS)
            self.assertEqual(read_attr_num('max_freq'), 950000000)
        elif soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(read_attr('available_frequencies'), mt8395_FREQS)
            self.assertEqual(read_attr_num('max_freq'), 880000000)
        elif soc == 'mt8365':
            self.assertEqual(read_attr('available_frequencies'), mt8365_FREQS)
            self.assertEqual(read_attr_num('max_freq'), 800000000)
        

    def test_max_freq_update(self):
        tmp = read_attr('max_freq')
        if soc == 'mt8365':
            self.assertTrue(write_attr('max_freq', '473000000'))
            self.assertEqual(read_attr_num('max_freq'), 473000000)  
        else:
            self.assertTrue(write_attr('max_freq', '500000000'))
            self.assertEqual(read_attr_num('max_freq'), 500000000)

        # Restore original value
        write_attr('max_freq', tmp)

    def test_freq_userspace_governor(self):
        tmp = read_attr('governor')
        self.assertTrue(write_attr('governor', 'userspace'))
        
        if soc == 'mt8365':
            self.assertTrue(write_attr('userspace/set_freq', '450000000'))
            self.assertEqual(read_attr_num('cur_freq'), 450000000)
        else:
            self.assertTrue(write_attr('userspace/set_freq', '515000000'))
            self.assertEqual(read_attr_num('cur_freq'), 515000000)

        write_attr('governor', tmp)

if __name__ == '__main__':
    unittest.main()
