#!/usr/bin/env python3

############################################################################
#Script Name    : S008001_gst_vdec_pipeline_check.py
#Description    : Sript for checking return value of Gstreamer pipeline
#                 with checksumsink
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

TEST_PATTERN_DIR = '/home/ubuntu/test/genio-linux-test/test_pattern/008_vdec'
def subcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    return ret

def runcmd(colorimetry,decoder):
    if decoder == 'vp8' or decoder == 'vp9':
        ret = subcmd(["gst-launch-1.0 -v filesrc location={2}/320x320-{1}-{0}.webm ! parsebin ! v4l2{1}dec ! v4l2convert ! checksumsink hash=0 sync=false > {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
    
    else:
        ret = subcmd(["gst-launch-1.0 -v filesrc location={2}/320x320-{1}-{0}.mp4 ! parsebin ! v4l2{1}dec ! v4l2convert ! video/x-raw,format={0} ! checksumsink hash=0 sync=false > {2}/out-320x320-{1}-{0}.md5".format(colorimetry,decoder,TEST_PATTERN_DIR)])
    return ret
    
class TestGstVdecRetVal(unittest.TestCase):

    def test_v4l2h264dec_i420(self):     
        ret = runcmd("I420","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h264dec_yu12(self):
        ret = runcmd("YV12","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h264dec_nv12(self):
        ret = runcmd("NV12","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h264dec_nv21(self):
        ret = runcmd("NV21","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h264dec_rgb(self):
        ret = runcmd("RGB","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)
    
    def test_v4l2h264dec_argb(self):
        ret = runcmd("ARGB","h264")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h265dec_i420(self):     
        ret = runcmd("I420","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h265dec_yu12(self):
        ret = runcmd("YV12","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h265dec_nv12(self):
        ret = runcmd("NV12","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h265dec_nv21(self):
        ret = runcmd("NV21","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2h265dec_rgb(self):
        ret = runcmd("RGB","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)
        
    def test_v4l2h265dec_argb(self):
        ret = runcmd("ARGB","h265")
        self.assertEqual(ret.returncode, 0, ret.stderr)
    
    def test_v4l2vp8dec_yuv(self):
        ret = runcmd("YUV","vp8")
        self.assertEqual(ret.returncode, 0, ret.stderr)

    def test_v4l2vp9dec_yuv(self):
        ret = runcmd("YUV","vp9")
        self.assertEqual(ret.returncode, 0, ret.stderr)

if __name__ == '__main__':
    
    unittest.main()
