#!/usr/bin/env python3

############################################################################
#Script Name    : S012001_mali_info.py.py
#Description    : Sript for checking GPU info (MALI) 
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

BASE = '/sys/class/misc/mali0/device'
mt8390_INFO = 'Mali-G57 3 cores r0p0 0x9091'
mt8395_INFO = 'Mali-G57 5 cores r0p1 0x9091'
mt8365_INFO = 'Mali-G52 1 cores r1p0 0x7402'


def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp
    
def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc
    
class TestCheckMaliInfo(unittest.TestCase):

    def test_sysfs_existence(self):
        self.assertTrue(os.path.isdir(BASE))

    def test_mali_info_correctness(self):
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(read_attr('gpuinfo'), mt8390_INFO)
        elif soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(read_attr('gpuinfo'), mt8395_INFO)
        if soc == 'mt8365':
            self.assertEqual(read_attr('gpuinfo'), mt8365_INFO)

if __name__ == '__main__':
    
    unittest.main()
