#!/usr/bin/env python3

############################################################################
#Script Name    : S014001_i2c.py
#Description    : Sript for testing i2C read/write
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    return ret
    
def read_attr(attr):
    path = os.path.join(BASE, attr)
    try:
        f = open(path)
        tmp = f.read().strip()
        f.close()
    except IOError:
        return ''

    return tmp

def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc  

def get_board_name():
    ret = subprocess.run(["fw_printenv | grep board_name"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    board_name = ret.stdout.split('=')[1].split('\n')[0]
    return board_name

board = get_board_name()

@unittest.skipIf(board == 'mt8390-tungsten_smarc', "test is not compatible with tungsten700 board")
    
class TestI2C(unittest.TestCase):
            
    def test_i2c_node_num_check(self):
        ret = runcmd(["i2cdetect -l | wc -l"])
        self.assertEqual(ret.returncode, 0)
        node_num = ret.stdout
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(int(node_num), 8)
        elif soc == 'mt8395':
            self.assertEqual(int(node_num), 5)
        elif soc == 'mt8365':
            self.assertEqual(int(node_num), 4)
        
        
    def test_i2c_read_attr(self):
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            ret = runcmd(["i2cget -y -f 3 0x55 0x00"])
            self.assertEqual(ret.returncode, 0)
            self.assertEqual(ret.stdout.split('\n')[0], '0x5a')
            
        elif soc == 'mt8395' or soc == 'mt8195':
            ret = runcmd(["i2cget -y -f 0 0x51 0x00"])
            self.assertEqual(ret.returncode, 0)
            self.assertEqual(ret.stdout.split('\n')[0], '0x5a')
            
        elif soc == 'mt8365':
            ret = runcmd(["i2cget -y -f 0 0x57 0x02"])
            self.assertEqual(ret.returncode, 0)
            self.assertEqual(ret.stdout.split('\n')[0], '0x92')
        
        
    def test_i2c_write_attr(self):
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            w_ret = runcmd(["i2cset -y -f 3 0x55 0x00 0x5b"])
            self.assertEqual(w_ret.returncode, 0)
            
            r_ret = runcmd(["i2cget -y -f 3 0x55 0x00"])
            self.assertEqual(r_ret.returncode, 0) 
            self.assertEqual(r_ret.stdout.split('\n')[0], '0x5b')
            
            # Set to default
            w_ret = runcmd(["i2cset -y -f 3 0x55 0x00 0x5a"])
            self.assertEqual(w_ret.returncode, 0)
        
        elif soc == 'mt8395' or soc == 'mt8195':
            w_ret = runcmd(["i2cset -y -f 0 0x51 0x00 0x5b"])
            self.assertEqual(w_ret.returncode, 0)
            
            r_ret = runcmd(["i2cget -y -f 0 0x51 0x00"])
            self.assertEqual(r_ret.returncode, 0) 
            self.assertEqual(r_ret.stdout.split('\n')[0], '0x5b')
            
            # Set to default
            w_ret = runcmd(["i2cset -y -f 0 0x51 0x00 0x5a"])
            self.assertEqual(w_ret.returncode, 0)
        
        elif soc == 'mt8365':
            w_ret = runcmd(["i2cset -y -f 0 0x57 0x02 0x93"])
            self.assertEqual(w_ret.returncode, 0)
            
            r_ret = runcmd(["i2cget -y -f 0 0x57 0x02"])
            self.assertEqual(r_ret.returncode, 0) 
            self.assertEqual(r_ret.stdout.split('\n')[0], '0x93')
            
            # Set to default
            w_ret = runcmd(["i2cset -y -f 0 0x57 0x02 0x92"])
            self.assertEqual(w_ret.returncode, 0)
                
if __name__ == '__main__':
    
    unittest.main()