#!/usr/bin/env python3

############################################################################
#Script Name    : S020001_gpio.py
#Description    : Sript for testing GPIO read/write  
#Author         : Joying Kuo
#Email          : joying.kuo@mediatek.com
###########################################################################

import os, sys, pathlib
import unittest
import subprocess

def runcmd(command):
    ret = subprocess.run(command,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8",timeout=2)
    return ret
    
def get_soc_name():
    ret = subprocess.run(["cat /proc/device-tree/compatible"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,encoding="utf-8")
    soc = ret.stdout.split(',')[1].split('-')[0]
    return soc

    
class TestGPIO(unittest.TestCase):
    
    def test_gpio_detect(self):
        ret = runcmd(["gpiodetect"])
        self.assertEqual(ret.returncode, 0)
        soc = get_soc_name()
        if soc == 'mt8390' or soc == 'mt8370':
            self.assertEqual(ret.stdout.split('\n')[0], 'gpiochip0 [pinctrl_paris] (190 lines)')
        elif soc == 'mt8395' or soc == 'mt8195':
            self.assertEqual(ret.stdout.split('\n')[0], 'gpiochip0 [pinctrl_paris] (165 lines)')
        elif soc == 'mt8365':
            self.assertEqual(ret.stdout.split('\n')[0], 'gpiochip0 [1000b000.pinctrl] (156 lines)')

        
    def test_gpio_read(self):    
        r_ret = runcmd(["gpioinfo gpiochip0"])
        self.assertEqual(r_ret.returncode, 0)
        self.assertEqual(r_ret.stdout.split('\n')[1].split()[4], 'input')
        
    def test_gpio_write(self): 
        soc = get_soc_name()
        if soc == 'mt8365':
            w_ret = runcmd(["gpioset gpiochip0 22=1"])
            self.assertEqual(w_ret.returncode, 0)
            
            r_ret = runcmd(["gpioinfo gpiochip0"])
            self.assertEqual(r_ret.returncode, 0)
            self.assertEqual(r_ret.stdout.split('\n')[23].split()[4], 'output')
            
            r_ret = runcmd(["gpioget gpiochip0 22"])
            self.assertEqual(r_ret.returncode, 0)
            
            r_ret = runcmd(["gpioinfo gpiochip0"])
            self.assertEqual(r_ret.returncode, 0)
            self.assertEqual(r_ret.stdout.split('\n')[23].split()[4], 'input')
        
        else:
            w_ret = runcmd(["gpioset gpiochip0 0=1"])
            self.assertEqual(w_ret.returncode, 0)
            
            r_ret = runcmd(["gpioinfo gpiochip0"])
            self.assertEqual(r_ret.returncode, 0)
            self.assertEqual(r_ret.stdout.split('\n')[1].split()[4], 'output')
            
            r_ret = runcmd(["gpioget gpiochip0 0"])
            self.assertEqual(r_ret.returncode, 0)
            
            r_ret = runcmd(["gpioinfo gpiochip0"])
            self.assertEqual(r_ret.returncode, 0)
            self.assertEqual(r_ret.stdout.split('\n')[1].split()[4], 'input')
        
        
if __name__ == '__main__':
    
    unittest.main()